.\" ========================================================
.TH XMLFIXUP 1 "[02-Dec-2003]" "1.01"
.\" ========================================================
.SH NAME
xmlfixup \(em Prettyprint XML documents or document fragments
.\" ========================================================
.SH SYNOPSIS
.B xmlfixup
[
.B \-\|\-?
]
[
.BI \-\|\-configfile " infile"
]
[
.B \-\|\-debug
]
.if n .ti +10n
[
.BI \-\|\-dumpconfig " outfile"
]
[
.B \-\|\-help
]
.if n .ti +10n
.if t .ti +\w'\fBxmlfixup\fP\ 'u
[
.BI \-\|\-indentation " n"
]
[
.BI \-\|\-level " n"
]
[
.B \-\|\-version
]
.if n .ti +10n
[
.BI \-\|\-width " n"
]
[
.B \-\|\-
]
.RI "[ " XML-file(s)
]
.\" ========================================================
.SH DESCRIPTION
.B xmlfixup
formats an XML input stream from one or more input files, or
from
.IR stdin ,
and writes a prettyprinted output stream on
.IR stdout ,
with tag environments indented according to user-specifiable
rules.
.PP
Unlike SGML and XML parsers,
.B xmlfixup
can be applied to incomplete XML documents, and to document
fragments, making it a convenient tool for reformatting XML
markup in a text-editor session, when the editor provides
for filtering text regions by an external program.
.PP
Suitable indentation usually helps to clarify document
structure dramatically, and can also reveal instances of
improper tag nesting.  However, a strict grammatical
analysis of XML documents with an SGML parser, such as
.BR nsgmls (1)
or
.BR sgmls (1),
or with an XML parser, such as
.BR xmllint (1)
or
.BR xmlwf (1),
to ensure well-formed XML is always advisable before
submitting an XML stream to other XML processing tools.
.PP
To achieve the requested linewidth goal,
.B xmlfixup
normally breaks the input stream at whitespace or a newline
when the next token would cause the requested linewidth to
be exceeded.  However, it will not do so at space inside an
XML tag, or where there is no existing breakpoint.  Thus, a
long punctuation-terminated sequence like
.RS
.nf
\&\fC<systemitem role="url">http://www.xml.org/long/path/</systemitem>.\fP
.fi
.RE
will
.I not
be broken, and may exceed the specified linewidth.
.\" ========================================================
.SH OPTIONS
.B xmlfixup
options can be prefixed with either one or two hyphens, and
can be abbreviated to any unique prefix.  Thus,
.BR \-v ,
.BR \-ver ,
and
.B \-\|\-version
are equivalent.
.PP
.\" --------------------------------------------------------
.TP \w'\fB\-\|\-dumpconfig\fP\ \fIoutfile\fP'u+3n
.B \-\|\-
All following command-line arguments are files, even if
they look like options.
.\" --------------------------------------------------------
.TP
.B \-\|\-?
Same as
.BR \-\|\-help .
.\" --------------------------------------------------------
.TP
.BI \-\|\-configfile " infile"
Supply alternate XML tag indentation rules, as described
in the
.B "CONFIGURATION FILES"
section.  If this option is specified more than once,
.B xmlfixup
uses the last one.  If it is not specified at all, then
.B xmlfixup
supplies a default set of rules suitable for a subset of
DocBook/XML markup.
.\" --------------------------------------------------------
.TP
.B \-\|\-debug
Write voluminous debug output on
.IR stderr .
.IP
This option may be abbreviated to
.BR \-d .
.\" --------------------------------------------------------
.TP
.BI \-\|\-dumpconfig " outfile"
Print the current XML tag indentation rules on
.IR outfile .
That file can be used on a subsequent run with the
.B \-\|\-configfile
option.
.IP
This option may be abbreviated to
.BR \-du .
.\" --------------------------------------------------------
.TP
.B \-\|\-help
Display a brief help message on
.IR stdout ,
giving a usage description, and then terminate immediately
with a success return code.
.\" --------------------------------------------------------
.TP
.BI \-\|\-indentation " n"
Set the number of indentation spaces for each logical tag
level to
.I n
(default: 2).
.\" --------------------------------------------------------
.TP
.BI \-\|\-level " n"
Set the initial logical tag level to
.I n
(default: 0).
.\" --------------------------------------------------------
.TP
.B \-\|\-version
Display the program version number and release date on
.IR stdout ,
and then terminate immediately with a success return code.
.\" --------------------------------------------------------
.TP
.BI \-\|\-width " n"
Set the output linewidth to
.I n
(default: 60).  A negative or zero value is treated as
infinite.
.\" ========================================================
.SH "CONFIGURATION FILES"
Because XML markup is entirely defined by the XML grammar
file that is loaded at the start of any document, unlike
HTML tags, XML tags do not form a small known set.
Consequently, an XML prettyprinter must be given rules that
describe the desired formatting of all tags used in the
input stream.
.PP
The default XML markup assumed by
.B xmlfixup
is a small subset of the DocBook/XML markup tags, but
alternative indentation rules for other tag sets can be
easily defined in a configuration file supplied as a
command-line argument.
.PP
The formatting required for documents in XML markup can be
specified by assigning XML tags to one of several formatting
classes, illustrated by the default configuration file
output with the
.B \-\|\-dumpconfig
option:
.PP
.RS
.nf
\&\fC# xmlfixup version 1.01 [02-Dec-2003]

Begin_Verbatim    : # clear existing list
Begin_Verbatim    : <screen>

Break_Before      : # clear existing list
Break_Before      : <colspec> <systemitem> <xref>

Empty_Line_After  : # clear existing list
Empty_Line_After  : </entry> </item> </listitem> </para> </row>
Empty_Line_After  : </screen> </tbody> </thead> </varlistentry>

Empty_Line_Before : # clear existing list
Empty_Line_Before : <entry> <item> <listitem> <para> <row>
Empty_Line_Before : <screen> <tbody> <thead> <varlistentry>
Empty_Line_Before : <xref>

End_Verbatim      : # clear existing list
End_Verbatim      : </screen>

No_Break_Before   : # clear existing list
No_Break_Before   : <footnote>

Ordinary          : # clear existing list
Ordinary          : <!> </citetitle> </command> </emphasis>
Ordinary          : </envvar> </filename> </firstterm>
Ordinary          : </indexterm> </literal> </option> </quote>
Ordinary          : </replaceable> </screen> </secondary>
Ordinary          : </subscript> </superscript> </systemitem>
Ordinary          : </term> </tertiary> </title> <?> <citetitle>
Ordinary          : <colspec> <command> <emphasis> <envvar>
Ordinary          : <filename> <firstterm> <footnoteref>
Ordinary          : <indexterm> <literal> <option> <primary>
Ordinary          : <quote> <replaceable> <screen> <secondary>
Ordinary          : <spanspec> <subscript> <superscript>
Ordinary          : <systemitem> <term> <tertiary> <title> <xref>\fP
.fi
.RE
.PP
Unlike in HTML and most SGML document types, markup tags in
XML are
.IR case-sensitive ,
and
.B xmlfixup
follows that practice.  DocBook/XML tags are all lowercase,
but tags for other XML document types may use mixed
lettercase.
.PP
In a configuration file, comments run from sharp to end of
line, and are removed before further processing.  Blank or
empty lines, and whitespace at start or end of line, are
ignored.
.PP
The formatting class is defined by the name before the
colon, which may optionally be surrounded by whitespace.
The colon is followed by a whitespace-separated list of zero
or more tags in that class.  An empty list clears the class
list, and otherwise, tags for repeated class names simply
augment the list for that class.
.PP
The \fCBegin_Verbatim\fP and \fCEnd_Verbatim\fP classes
contain tag environments that must be copied verbatim,
without any change whatsoever in indentation or spacing.
.PP
The \fCBreak_Before\fP class contains tags which should
begin on a new line, and thus, require a line break
preceding the tag.
.PP
The \fCNo_Break_Before\fP class contains tags, like those
for footnotes, which must be attached to preceding text, and
thus, must not be preceded by a line break.  However, they
are otherwise treated like normal tags with indented bodies.
Existing whitespace preceding the open tag will not be
eliminated.
.PP
The \fCEmpty_Line_After\fP and \fCEmpty_Line_Before\fP
classes contain tags that should appear alone on their line,
with an empty line after or before them, respectively.
.PP
The \fCOrdinary\fP class contains tags that behave like
ordinary text, and thus require no special formatting; in
particular, they do not cause line breaks.
.PP
Two kinds of tags receive special treatment.  Formatter
directives of the form \fC<?name?>\fP are classed according
to the reduced form \fC<?>\fP.  Comments (\fC<!--
\&.\|.\|.-->\fP) and SGML directives
(\fC<!NAME \&.\|.\|.>\fP) are classed according to the
reduced form \fC<!>\fP.  Consequently, only those reduced
forms need be used in the classification rules.
.PP
Tags may belong to more than one class.
.PP
Tag attributes are ignored when tags are classified: thus,
\fC<chapter id="mybook-ch-3">\fP is formatted just like
\fC<chapter>\fP is.
.PP
Outside the seven classes, all other tags are assumed to
define environments with indented bodies.  This unnamed
class thus serves as a catch-all for an unbounded set of
unclassified tags. The
.I open
tag (\fC<name>\fP) appears on a line by itself at the
current indentation level, the level is incremented by one
for the body, and the
.I close
tag (\fC</name>\fP) appears on a line by itself at the same
level as the open tag:
.PP
.RS
.nf
\&\fC<tag>
  Text text text.\|.\|.
  Text text text.\|.\|.
  <tag>
    More text text text.\|.\|.
    <tag>
      Even more text text text.\|.\|.
    </tag>
  </tag>
  Text text text.\|.\|.
</tag>\fP
.fi
.RE
.PP
.B xmlfixup
adjusts indentation levels according to whether the tag is
an open tag or a close tag: it issues a warning on
.IR stderr ,
and also inside an XML comment on
.IR stdout ,
if the tag names do not match.  If this happens, it means
that either the XML input is not well-formed, or else there
is an inconsistency in the markup rules in a user-supplied
configuration file.
.\" ========================================================
.SH "SEE ALSO"
.BR gmat (1),
.BR html-check (1),
.BR html-ncheck (1),
.BR html-pretty (1),
.BR html-spam (1),
.BR nsgmls (1),
.BR sgmlnorm (1),
.BR sgmls (1),
.BR xmlcatalog (1),
.BR xmllint (1),
.BR xmlwf (1).
.\"=========================================================
.SH BUGS
The default formatting matches that found useful in a single
computer-related book project.  For other documents, even
ones using more of the DocBook/XML markup scheme, it will
almost certainly be necessary to supply a configuration file
to define how you want the markup handled: use the
.B \-\|\-dumpconfig
option to get a starting configuration file that you can
then modify as needed.
.PP
.B xmlfixup
does not currently offer any control over the formatting of
the internals of SGML comments, declarations, and directives
(all of which look like \fC<!.\|.\|.>\fP), or of processor
directives (which look like \fC<?.\|.\|.>\fP).
.PP
It may even prove necessary to extend
.B xmlfixup
with additional tag classes to handle a wider variety of XML
document types.
.PP
.B xmlfixup
does not know anything about SGML tag minimization (use of
unnamed end tags \fC</>\fP, and omission of end tags where
their implied positions can be determined by the SGML parser
from the markup grammar).  A tag normalizer, such as
.BR sgmlnorm (1)
or
.BR spam (1),
would likely be needed to standardize SGML input before
.B xmlfixup
could format it sensibly.
.PP
.B xmlfixup
does not currently offer an option to ignore lettercase in
tags, making it less useful for HTML and SGML documents with
inconsistent lettercase in tags.  A tag normalizer can fix
such problems.   For HTML files,
.BR html-pretty (1)
has extensive knowledge of HTML variants, and can do an
excellent job of prettyprinting such files.
.\"=========================================================
.SH "FURTHER READING"
These books may be helpful in understanding DocBook/XML
markup, and SGML and XML markup in general:
.RS
.TP \w'\(bu'u+1n
\(bu
Peter Flynn,
.I "Understanding SGML and XML tools: practical programs"
.IR "for handling structured text" ,
Kluwer, 1998, ISBN 0-7923-8169-6.
.TP
\(bu
Charles F. Goldfarb and Yuri Rubinsky,
.IR "The SGML handbook" ,
Clarendon Press, 1990, ISBN 0-19-853737-9.
.TP
\(bu
Charles F. Goldfarb and Paul Prescod,
.IR "Charles F. Goldfarb's XML handbook" ,
fourth edition, Prentice-Hall PTR, 2001, ISBN 0-13-065198-2.
.TP
\(bu
Michel Goossens and Sebastian Rahtz,
.IR "The LaTeX Web companion: integrating TeX, HTML, and XML" ,
Addison-Wesley Longman, 1999, ISBN 0-201-43311-7.
.TP
\(bu
Norman Walsh and Leonard Muellner,
.IR "DocBook: the definitive guide" ,
O'Reilly & Associates, 1999, ISBN 1-56592-580-7.
.RE
.PP
References to hundreds of other books on HTML, SGML, and XML
can be found in the bibliographies at:
.RS
.nf
\&\fChttp://www.math.utah.edu/pub/tex/bib/index-table.html#sgml
http://www.math.utah.edu/pub/tex/bib/index-table.html#sgml2000\fP
.fi
.RE
.\"=========================================================
.SH AUTHOR
.nf
Nelson H. F. Beebe
University of Utah
Department of Mathematics, 110 LCB
155 S 1400 E RM 233
Salt Lake City, UT 84112-0090
Tel: +1 801 581 5254
FAX: +1 801 581 4148
Email: \fCbeebe@math.utah.edu\fP, \fCbeebe@acm.org\fP,
       \fCbeebe@computer.org\fP (Internet)
WWW URL: \fChttp://www.math.utah.edu/~beebe\fP
.fi
.\"=========================================================
.SH AVAILABILITY
.B xmlfixup
is freely available; its master distribution can be found at
.PP
.RS
.nf
\&\fCftp://ftp.math.utah.edu/pub/xmlfixup/\fP
.fi
.RE
.PP
in the file
.I xmlfixup-x.yy.tar.gz
where
.I x.yy
is the current version.  Other distribution formats are
usually available in the same location.
.\"==============================[The End]==============================
.\" This is for GNU Emacs file-specific customization:
.\" Local Variables:
.\" fill-column: 60
.\" End:
